#ifndef INCLUDE_MENUITEM_DEF
#define INCLUDE_MENUITEM_DEF
#endif

#include <qmenudata.h>
#include <qwindowsstyle.h>
#include <qpalette.h>
#include <qbitmap.h>
#include <qtabbar.h>
#include <qpointarray.h>

#include <kapp.h>
#include <kdrawutil.h>
#include <kglobalsettings.h>
#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>

#include "dotNETStyle.h"

extern "C" {
	KStyle *allocate() {
		return new dotNETStyle;
	} int minor_version() {
		return 0;
	}
	int major_version() {
		return 8;
	}
	const char *description() {
		return (i18n("Based on screenshots of OfficeXP").utf8());
	}
}

dotNETStyle::dotNETStyle()
:  KStyle()
{
}

dotNETStyle::~dotNETStyle()
{
	// Empty.
}

void dotNETStyle::polish(QApplication * app)
{
	KStyle::polish(app);
}

void dotNETStyle::polish(QPalette & pal)
{
	KStyle::polish(pal);
}

void dotNETStyle::unPolish(QApplication * app)
{
	KStyle::unPolish(app);
}

void dotNETStyle::polish(QWidget * w)
{
	if (w->isTopLevel())
		return;

	if ( /*w->inherits("QButton") || */ w->inherits("QComboBox")) {
		w->installEventFilter(this);
		QPalette pal = w->palette();
		// we use this as a flag since it's otherwise unused.
		pal.setColor(QColorGroup::Mid,
			     pal.active().color(QColorGroup::Background));
		pal.setColor(QColorGroup::Shadow,
			     pal.active().color(QColorGroup::Light));
		w->setPalette(pal);
	}
}

void dotNETStyle::unPolish(QWidget * w)
{
	KStyle::unPolish(w);
}

void dotNETStyle::drawFocusRect(QPainter * p,
				const QRect & r,
				const QColorGroup & cg, const QColor * bg)
{
	//	Hmm. It appears that this function actually isn't called. Ever.
	if (!bg) {
		p->fillRect(r, cg.highlight());
	//	p->drawWinFocusRect(r);
	} else {
		p->fillRect(r, cg.highlight());
	//	p->drawWinFocusRect(r, *bg);
	}
}

void dotNETStyle::drawButton
    (QPainter * p,
     int x,
     int y,
     int w,
     int h, const QColorGroup & g, bool sunken, const QBrush * fill)
{
	if (sunken) {
		p->save();
		p->setBrush(g.highlight());
		p->setPen(g.highlight().dark());
		p->drawRect(x, y, w, h);
		p->restore();
	} else {
		qDrawWinShades(p, x, y, w, h,
		               g.midlight(), g.highlight().dark(), g.light(),
		               g.dark(), fill);
	}
//	KStyle::drawButton(p, x, y, w, h, g, sunken, fill);
}

QRect dotNETStyle::buttonRect(int x, int y, int w, int h)
{
	return KStyle::buttonRect(x, y, w, h);
}

void dotNETStyle::drawBevelButton
    (QPainter * p,
     int x,
     int y,
     int w,
     int h, const QColorGroup & g, bool sunken, const QBrush * fill) {
	KStyle::drawBevelButton(p, x, y, w, h, g, sunken, fill);
}

void dotNETStyle::drawPushButton(QPushButton * b, QPainter * p)
{
	KStyle::drawPushButton(b, p);
}

void dotNETStyle::drawPushButtonLabel(QPushButton * b, QPainter * p)
{
	KStyle::drawPushButtonLabel(b, p);
}

void dotNETStyle::drawScrollButton(QPainter * p,
				   int x,
				   int y,
				   int w,
				   int h,
				   const QColorGroup & g,
				   bool sunken, const QBrush * fill)
{
	if (sunken) {
		p->save();
		p->setBrush(g.highlight());
		p->setPen(g.highlight().dark());
		p->drawRect(x, y, w, h);
		p->restore();
	} else {
		qDrawWinShades(p, x, y, w, h,
		               g.midlight(), g.highlight().dark(), g.light(),
		               g.dark(), fill);
	}
}

void dotNETStyle::drawScrollBarControls(QPainter * p,
					const QScrollBar * sb,
					int sliderStart,
					uint controls, uint activeControl)
{
#define HORIZONTAL	(sb->orientation() == QScrollBar::Horizontal)
#define VERTICAL	!HORIZONTAL
#define MOTIF_BORDER	2
#define SLIDER_MIN	9	// wtf does this have to do with motif? -qt coder
	// Heh. Qt coders r0x0r!
#ifndef QT_NO_SCROLLBAR
#define ADD_LINE_ACTIVE ( activeControl == AddLine )
#define SUB_LINE_ACTIVE ( activeControl == SubLine )
	QColorGroup g = sb->colorGroup();

	int sliderMin, sliderMax, sliderLength, buttonDim;
	scrollBarMetrics(sb, sliderMin, sliderMax, sliderLength,
			 buttonDim);

	if (sliderStart > sliderMax) {	// sanity check
		sliderStart = sliderMax;
	}

	int b = 0;
	int dimB = buttonDim;
	QRect addB;
	QRect subB;
	QRect addPageR;
	QRect subPageR;
	QRect sliderR;
	int addX, addY, subX, subY;
	int length = HORIZONTAL ? sb->width() : sb->height();
	int extent = HORIZONTAL ? sb->height() : sb->width();

	if (HORIZONTAL) {
		subY = addY = (extent - dimB) / 2;
		subX = b;
		addX = length - dimB - b;
	} else {
		subX = addX = (extent - dimB) / 2;
		subY = b;
		addY = length - dimB - b;
	}

	subB.setRect(subX, subY, dimB, dimB);
	addB.setRect(addX, addY, dimB, dimB);

	int sliderEnd = sliderStart + sliderLength;
	int sliderW = extent - b * 2;
	if (HORIZONTAL) {
		subPageR.setRect(subB.right() + 1, b,
				 sliderStart - subB.right() - 1, sliderW);
		addPageR.setRect(sliderEnd, b, addX - sliderEnd, sliderW);
		sliderR.setRect(sliderStart, b, sliderLength, sliderW);
	} else {
		subPageR.setRect(b, subB.bottom() + 1, sliderW,
				 sliderStart - subB.bottom() - 1);
		addPageR.setRect(b, sliderEnd, sliderW, addY - sliderEnd);
		sliderR.setRect(b, sliderStart, sliderW, sliderLength);
	}

	bool maxedOut = (sb->maxValue() == sb->minValue());
	if (controls & AddLine) {
//      qDrawWinPanel( p, addB.x(), addB.y(), addB.width(), addB.height(), g, ADD_LINE_ACTIVE, &g.brush( QColorGroup::Button ) );
		drawScrollButton(p, addB.x(), addB.y(), addB.width(),
				 addB.height(), g, ADD_LINE_ACTIVE,
				 &g.brush(QColorGroup::Button));
		drawArrow(p, VERTICAL ? DownArrow : RightArrow,
			  ADD_LINE_ACTIVE, addB.x() + 2, addB.y() + 2,
			  addB.width() - 4, addB.height() - 4, g,
			  !maxedOut);
	}
	if (controls & SubLine) {
//      qDrawWinPanel( p, subB.x(), subB.y(), subB.width(), subB.height(), g, SUB_LINE_ACTIVE, &g.brush( QColorGroup::Button )  );
		drawScrollButton(p, subB.x(), subB.y(), subB.width(),
				 subB.height(), g, SUB_LINE_ACTIVE,
				 &g.brush(QColorGroup::Button));
		drawArrow(p, VERTICAL ? UpArrow : LeftArrow,
			  SUB_LINE_ACTIVE, subB.x() + 2, subB.y() + 2,
			  subB.width() - 4, subB.height() - 4, g,
			  !maxedOut);
	}
	QBrush br =
	    g.brush(QColorGroup::Light).pixmap()?
	    g.brush(QColorGroup::Light) : QBrush(g.light(), Dense4Pattern);
	p->setBrush(br);
	p->setPen(NoPen);
	p->setBackgroundMode(OpaqueMode);
	if (maxedOut) {
		p->drawRect(sliderR);
	} else {
		if ((controls & SubPage && SubPage == activeControl) ||
		    (controls & AddPage && AddPage == activeControl)) {
			QBrush b = p->brush();
			QColor c = p->backgroundColor();
//          p->fillRect( AddPage == activeControl? addPageR : subPageR, g.fillDark() );
			p->setBackgroundColor(g.dark());
			p->setBrush(QBrush(g.shadow(), Dense4Pattern));
			p->drawRect(AddPage ==
				    activeControl ? addPageR : subPageR);
			p->setBackgroundColor(c);
			p->setBrush(b);
		}
		if (controls & SubPage && SubPage != activeControl)
			p->drawRect(subPageR);
		if (controls & AddPage && AddPage != activeControl)
			p->drawRect(addPageR);
		if (controls & Slider) {
			QPoint bo = p->brushOrigin();
			p->setBrushOrigin(sliderR.topLeft());
			drawScrollButton(p, sliderR.x(), sliderR.y(),
			                 sliderR.width(), sliderR.height(),
			                 g, sb->draggingSlider(),
			                 &g.brush (QColorGroup::Button));
			   /*
			qDrawWinPanel(p, sliderR.x(), sliderR.y(),
				      sliderR.width(),
				      sliderR.height(), g, FALSE,
				      &g.brush(QColorGroup::Button));
			*/
			p->setBrushOrigin(bo);
		}
	}
	// ### perhaps this should not be able to accept focus if maxedOut?
	if (sb->hasFocus() && (controls & Slider))
		drawFocusRect(p, QRect(sliderR.x() + 2, sliderR.y() + 2,
				       sliderR.width() - 5,
				       sliderR.height() - 5), g,
			               &sb->backgroundColor());
#endif
//  QWindowsStyle::drawScrollBarControls(p,sb,sliderStart,controls,activeControl);
//  KStyle::drawScrollBarControls(p, sb, sliderStart, controls, activeControl);
}

QStyle::ScrollControl
    dotNETStyle::scrollBarPointOver(const QScrollBar * sb, int sliderStart,
				    const QPoint & p)
{
	if (!sb->rect().contains(p))
		return (NoScroll);

	int sliderMin, sliderMax, sliderLength, buttonDim;
	scrollBarMetrics(sb, sliderMin, sliderMax, sliderLength,
			 buttonDim);

	if (sb->orientation() == QScrollBar::Horizontal) {
		int x = p.x();
		if (x < buttonDim)
			return (SubLine);
		else if (x < sliderStart)
			return (SubPage);
		else if (x < sliderStart + sliderLength)
			return (Slider);
		else if (x < sliderMax + sliderLength)
			return (AddPage);

		return (AddLine);
	} else {
		int y = p.y();
		if (y < buttonDim)
			return (SubLine);
		else if (y < sliderStart)
			return (SubPage);
		else if (y < sliderStart + sliderLength)
			return (Slider);
		else if (y < sliderMax + sliderLength)
			return (AddPage);

		return (AddLine);
	}
}

void dotNETStyle::scrollBarMetrics(const QScrollBar * sb, int &sliderMin,
				   int &sliderMax, int &sliderLength,
				   int &buttonDim)
{
	int maxLength;
	bool horizontal = sb->orientation() == QScrollBar::Horizontal;
	int len = (horizontal) ? sb->width() : sb->height();
	int extent = (horizontal) ? sb->height() : sb->width();
	int b = 0;		// border around sbar

	if (len > (extent - 1) * 2)
		buttonDim = extent - b * 2;
	else
		buttonDim = (len - b * 2) / 2 - 1;

	sliderMin = buttonDim + b;
	maxLength = len - b * 2 - buttonDim * 2;

	if (sb->maxValue() == sb->minValue()) {
		sliderLength = maxLength;
	} else {
		sliderLength = (sb->pageStep() * maxLength) /
		    (sb->maxValue() - sb->minValue() + sb->pageStep());
		uint range = sb->maxValue() - sb->minValue();
		if (sliderLength < 22 || range > INT_MAX / 2)
			sliderLength = 22;
		if (sliderLength > maxLength)
			sliderLength = maxLength;
	}

	sliderMax = sliderMin + maxLength - sliderLength;
}

QSize dotNETStyle::indicatorSize() const
{
	return KStyle::indicatorSize();
}

void dotNETStyle::drawIndicator(QPainter * p,
				int x,
				int y,
				int w,
				int h, const QColorGroup & g, int state,
				bool down, bool enabled)
{
	int l;
	p->save();

	if (w != h) {
		l = (w < h ? w : h);
	} else {
		l = h = w;
	}
	if (l % 2) {
		--l;
	}

	const int il = l - 6;
	const int ix = x + ((w - (il)) / 2);
	const int iy = y + ((h - (il)) / 2);

	const int ol = l - 4;
	const int ox = x + ((w - (ol)) / 2);
	const int oy = y + ((h - (ol)) / 2);

	if (enabled) {
		if (down)
			p->setPen(g.highlight());
		else
			p->setPen(g.dark());
	} else {
		p->setPen(g.mid());
	}

	p->fillRect(x, y, w, h, g.background());
	p->fillRect(ix, iy, il, il, g.background());
	p->drawRect(ox, oy, ol, ol);

	if (state != QButton::Off) {
		p->fillRect(ix, iy, il, il,
			    enabled ? g.highlight() : g.mid());
		if (state == QButton::NoChange) {
			p->fillRect(ix, iy, il, il, g.background());
		}
	}

	p->restore();
}

QSize dotNETStyle::exclusiveIndicatorSize() const
{
	return KStyle::exclusiveIndicatorSize();
}

void dotNETStyle::drawExclusiveIndicator
    (QPainter * p,
     int x,
     int y,
     int w,
     int h, const QColorGroup & g, bool on, bool down, bool enabled) {
	p->save();

	p->fillRect(x, y, w, h, g.background());

	if (enabled) {
		p->setPen(down ? g.highlight() : g.dark());
	} else {
		p->setPen(g.mid());
	}

	p->setBrush(g.brush(QColorGroup::Background));

	// Avoid misshapen ellipses. Qt or X bug ? Who knows...

	if (0 == w % 2)
		--w;

	if (0 == h % 2)
		--h;

	p->drawEllipse(x, y, w, h);

	if (on) {
		p->setPen(enabled ? g.highlight() : g.mid());
		p->setBrush(enabled ? g.highlight() : g.mid());
		p->drawEllipse(x + 3, y + 3, w - 6, h - 6);
	}

	p->restore();
}

void dotNETStyle::drawIndicatorMask
    (QPainter * p, int x, int y, int w, int h, int) {
	int l;
	(w != h ? (l = (w < h ? w : h)) : (l = h = w));

	if (l % 2) {
		--l;
	}

	p->fillRect(x + ((w - (l - 4)) / 2) + 1, y + ((h - (l - 4)) / 2),
		    l - 4, l - 4, Qt::color1);
}

void dotNETStyle::drawExclusiveIndicatorMask
    (QPainter * p, int x, int y, int w, int h, bool) {
	if (0 == w % 2)
		--w;

	if (0 == h % 2)
		--h;

	p->setPen(Qt::color1);
	p->setBrush(Qt::color1);
	p->drawEllipse(x + 1, y, w, h);
}

void dotNETStyle::drawComboButton
    (QPainter * p,
     int x,
     int y,
     int w,
     int h,
     const QColorGroup & g, bool sunken,
     bool, bool, const QBrush *) {
	static const unsigned char downarrow_bits[] = {
		0x7f, 0xbe, 0x9c, 0x08, 0x00, 0x00, 0x00, 0x28,
		0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0xb8
	};

	static const unsigned int handle_width = 13;
	static const unsigned int handle_offset = handle_width + 1;

	QBitmap downArrow = QBitmap(7, 4, downarrow_bits, true);
	downArrow.setMask(downArrow);

	p->fillRect(x + 1, y + 1, w - 2, h - 2, g.light());

	p->setPen(g.background().light());
	p->drawRect(x, y, w, h);
	p->setPen(g.background().dark());
	p->drawLine(x, y, w, y);
	p->drawLine(x, y, x, h);
	p->setPen(g.shadow());
	p->drawRect(w - handle_offset - 1, y+1, handle_width + 2, h - 1);

	p->fillRect(w - handle_offset, y + 2, handle_width, h - 3,
		    g.mid());
/*
	p->setPen (g.mid().dark());
	p->drawRect (w - handle_offset, y + 1, handle_width, h - 2);
	p->setPen (g.mid().light());
	p->drawLine (w - handle_offset, y + 1, w - 2, y + 1);
	p->drawLine (w - handle_offset, y + 1, w - handle_offset, h - 2);

	p->fillRect(x, y, w, h, g.light());
	p->fillRect(w - handle_offset, y + 2, handle_width, h - 4, g.mid());
	p->setPen(g.shadow());
	p->drawRect(x, y, w, h);
	p->drawRect(w - handle_offset, y, handle_width + 1, h);
*/
	if (sunken) {
		p->setPen(g.highlightedText());
	} else {
		p->setPen(g.text());
	}

	QRect cr(w - handle_offset, y + 2, handle_width, h - 4);
	QRect pmr(0, 0, 7, 4);
	pmr.moveCenter(cr.center());
	p->drawPixmap(pmr.topLeft(), downArrow);
}

QRect dotNETStyle::comboButtonRect(int x, int y, int w, int h)
{
	return KStyle::comboButtonRect(x, y, w, h);
}

QRect dotNETStyle::comboButtonFocusRect(int x, int y, int w, int h)
{
	return KStyle::comboButtonFocusRect(x, y, w, h);
}

int dotNETStyle::sliderLength() const
{
	return KStyle::sliderLength();
}

void dotNETStyle::drawSliderGroove
    (QPainter * p,
     int x,
     int y, int w, int h, const QColorGroup & g, QCOORD c, Orientation o) {
	KStyle::drawSliderGroove(p, x, y, w, h, g, c, o);
}

void dotNETStyle::drawArrow
    (QPainter * p,
     Qt::ArrowType type,
     bool down,
     int x,
     int y,
     int w,
     int h, const QColorGroup & g, bool enabled, const QBrush * fill) {
	KStyle::drawArrow(p, type, down, x, y, w, h, g, enabled, fill);
}

void dotNETStyle::drawSlider
    (QPainter * p,
     int x,
     int y,
     int w,
     int h,
     const QColorGroup & g,
     Orientation o, bool tickAbove, bool tickBelow) {
	KStyle::drawSlider(p, x, y, w, h, g, o, tickAbove, tickBelow);
}

void dotNETStyle::drawKToolBar
    (QPainter * p,
     int x,
     int y,
     int w, int h, const QColorGroup &, KToolBarPos, QBrush * fill) {
	QPen oldPen = p->pen();

	if (fill) {
		QBrush oldBrush = p->brush();
		p->setBrush(*fill);
		p->setPen(NoPen);
		p->drawRect(x, y, w, h);
		p->setBrush(oldBrush);
	}

	p->setPen(oldPen);
//  KStyle::drawKToolBar(p, x, y, w, h, g, pos, fill);
}

void dotNETStyle::drawKBarHandle
    (QPainter * p,
     int x,
     int y,
     int w, int h, const QColorGroup & g, KToolBarPos pos,
     QBrush * fill)
{
	int k;
	p->save();
	if (fill) {
		p->setBrush(*fill);
		p->setPen(NoPen);
		p->drawRect(x, y, w, h);
		p->setBrush(g.mid());
		if(pos == 2 || pos == 3) {
			for (k = 4; k < h - 4; k += 3) {
				p->drawRect(x + 2, k, w - 4, 1);
			}
		} else {
			for (k = 4; k < w - 4; k += 3) {
				p->drawRect(k, y, 1, h - 4);
			}
		}
	}
	p->restore();
}

void dotNETStyle::drawKMenuBar
    (QPainter * p,
     int x, int y, int w, int h, const QColorGroup & g, bool, QBrush *) {
	p->fillRect(x, y, w, h, g.brush(QColorGroup::Background));
}

void dotNETStyle::drawKToolBarButton
    (QPainter * p,
     int x,
     int y,
     int w,
     int h,
     const QColorGroup & g,
     bool sunken,
     bool raised,
     bool enabled,
     bool popup,
     KToolButtonType icontext,
     const QString & btext,
     const QPixmap * pixmap, QFont * font, QWidget *) {
	int dx, dy;

	QFontMetrics fm(*font);

	if (raised || sunken) {
		if (raised) {
			p->setPen(g.highlight().dark());
			p->drawRect(x, y, w, h);
			//drawFullShadeButton(p, x+1, y+1, w-2, h-2, g, sunken);
			p->fillRect(x + 1, y + 1, w - 2, h - 2,
				    g.highlight());
		}
		if (sunken) {
			p->setPen(g.highlight().dark());
			p->drawRect(x, y, w, h);
			//drawFullShadeButton(p, x+1, y+1, w-2, h-2, g, sunken);
			p->fillRect(x + 1, y + 1, w - 2, h - 2,
				    g.highlight());
		}
	} else {
		p->fillRect(x, y, w, h, g.brush(QColorGroup::Background));
	}

	p->setPen(g.buttonText());

	if (icontext == Icon) {	// icon only
		if (pixmap) {
			dx = (w - pixmap->width()) / 2;
			dy = (h - pixmap->height()) / 2;
			if (raised) {
				KPixmap kshade = KPixmap(*pixmap);
				dx += 1;
				dy += 1;
				p->drawPixmap(x + dx, y + dy,
					      KPixmapEffect::toGray(kshade,
								    true));
				dx -= 2;
				dy -= 2;
			}
			if (sunken) {
				dx = (w - pixmap->width()) / 2;
				dy = (h - pixmap->height()) / 2;
				p->fillRect(x + 1, y + 1, w - 2, h - 2,
					    g.highlight());
			}
			p->drawPixmap(x + dx, y + dy, *pixmap);
		}
	} else if (icontext == IconTextRight) {	// icon and text (if any)
		if (pixmap) {
			dx = 4;
			dy = (h - pixmap->height()) / 2;
			if (raised) {
				KPixmap kshade = KPixmap(*pixmap);
				dx += 1;
				dy += 1;
				p->drawPixmap(x + dx, y + dy,
					      KPixmapEffect::toGray(kshade,
								    true));
				dx -= 2;
				dy -= 2;
			}
			if (sunken) {
				dx = 4;
				dy = (h - pixmap->height()) / 2;
				p->fillRect(x + 1, y + 1, w - 2, h - 2,
					    g.highlight());
			}
			p->drawPixmap(x + dx, y + dy, *pixmap);
		}

		if (!btext.isNull()) {
			int tf = AlignVCenter | AlignLeft;
			if (pixmap)
				dx = 4 + pixmap->width() + 2;
			else
				dx = 4;
			dy = 0;
			if (font)
				p->setFont(*font);
			p->setPen(g.text());
			if (raised || sunken)
				p->setPen(g.highlightedText());
			p->drawText(x + dx, y + dy, w - dx, h, tf, btext);
		}
	} else if (icontext == Text) {	// only text, even if there is a icon
		if (!btext.isNull()) {
			int tf = AlignVCenter | AlignLeft;
			if (!enabled)
				p->setPen(g.dark());
			dx = (w - fm.width(btext)) / 2;
			dy = (h - fm.lineSpacing()) / 2;
			if (sunken) {
				++dx;
				++dy;
			}
			if (font)
				p->setFont(*font);
			p->setPen(g.text());
			if (raised)
				p->setPen(g.highlightedText());
			p->drawText(x + dx, y + dy, fm.width(btext),
				    fm.lineSpacing(), tf, btext);
		}
	} else if (icontext == IconTextBottom) {
		if (pixmap) {
			dx = (w - pixmap->width()) / 2;
			dy = (h - fm.lineSpacing() - pixmap->height()) / 2;
			if (sunken) {
				++dx;
				++dy;
			}
			p->drawPixmap(x + dx, y + dy, *pixmap);
		}
		if (!btext.isNull()) {
			int tf = AlignBottom | AlignHCenter;
			dy = pixmap->height();
			dx = 2;

			if (sunken) {
				++dx;
				++dy;
			}
			if (font)
				p->setFont(*font);
			if (raised)
				p->setPen(KGlobalSettings::
					  toolBarHighlightColor());
			p->drawText(x, y, w, h - 3, tf, btext);
		}
	}
	if (popup) {
		if (enabled)
			qDrawArrow(p, Qt::DownArrow, Qt::WindowsStyle,
				   false, w - 5, h - 5, 0, 0, g, true);
		else
			qDrawArrow(p, Qt::DownArrow, Qt::WindowsStyle,
				   false, w - 5, h - 5, 0, 0, g, false);
	}
}

void dotNETStyle::drawKMenuItem(QPainter * p,
				int x,
				int y,
				int w,
				int h,
				const QColorGroup & g,
				bool active, QMenuItem * mi, QBrush *)
{
	if (p->font() == KGlobalSettings::generalFont())
		p->setFont(KGlobalSettings::menuFont());

	if (active) {
		p->setPen(g.highlight().dark());
		p->drawRect(x, y, w, h);
		p->fillRect(x + 1, y + 1, w - 2, h - 2, g.highlight());

		QString s = mi->text();
		int t = s.find('\t');
		const int text_flags =
		    AlignVCenter | ShowPrefix | DontClip | SingleLine;
		p->setPen(g.highlightedText());
		p->drawText(x + 5, y + 3, w - 10, h - 6, text_flags, s, t);
	} else {
		QApplication::style().drawItem(p, x, y, w, h,
					       AlignCenter | ShowPrefix |
					       DontClip | SingleLine, g,
					       mi->isEnabled(),
					       mi->pixmap(), mi->text(),
					       -1, &g.buttonText());
	}
//  KStyle::drawKMenuItem(p, x, y, w, h, g, active, mi, fill);
}

void dotNETStyle::drawPopupMenuItem
    (QPainter * p,
     bool checkable,
     int maxpmw,
     int tab,
     QMenuItem * mi,
     const QPalette & pal,
     bool act, bool enabled, int x, int y, int w, int h) {
	static const int motifItemFrame = 0;
	static const int motifItemHMargin = 7;
	static const int motifItemVMargin = 7;
	static const int motifArrowHMargin = 6;
	static const int windowsRightBorder = 12;

	bool dis = !enabled;
	QColorGroup itemg = dis ? pal.disabled() : pal.active();

	maxpmw = QMAX(maxpmw, 20);
	int checkcol = maxpmw;

	if (p->font() == KGlobalSettings::generalFont())
		p->setFont(KGlobalSettings::menuFont());

	if (!mi) {
		return;
	}

	if (mi->isSeparator()) {
		p->fillRect(x, y, w, h, itemg.midlight());
		p->setPen(itemg.mid());
		p->drawLine(x + 28, y + 1, x + w, y + 1);
		p->fillRect(x, y, 22, h, itemg.mid());
		return;
	}

	if (act && enabled) {
		// draw highlight around current menu item
		QPen oldPen = p->pen();

		p->setPen(itemg.highlight().dark());
		p->setBrush(itemg.highlight());
		p->drawRect(x, y, w, h);

		p->setPen(oldPen);
	} else {
		p->fillRect(x, y, w, h, itemg.midlight());
		p->fillRect(x, y, 22, h, itemg.mid());
	}
	if (mi->isChecked() && !mi->iconSet()) {
	// draw check mark
		int w2 = checkcol - 2;
		int h2 = h - 2;

		const int ol = 10;
		const int ox = x + 2 + ((w2 - ol) / 2);
		const int oy = y + 1 + ((h2 - ol) / 2);
		if (w2 < 10 || h2 < 10)
			return;

		p->save();
		p->setPen(itemg.dark());
		mi->isChecked() ? p->setBrush(itemg.highlight()) :
			p->setBrush(itemg.background());
		p->drawRect(ox, oy, ol, ol);
		p->restore();
	//      drawIndicator(p, x + 2, y + 1, w2, h2, itemg, TRUE, FALSE,
	//                    enabled);
	} else if (mi->iconSet()) {
	// draw associated icon
		QIconSet::Mode mode =
		    dis ? QIconSet::Disabled : QIconSet::Normal;
		if (!dis)
			mode = QIconSet::Active;
		QPixmap pixmap =
		    mi->iconSet()->pixmap(QIconSet::Small, mode);
		int pixw = pixmap.width();
		int pixh = pixmap.height();

		QRect cr(x, y, checkcol, h);
		QRect pmr(0, 0, pixw, pixh);
		pmr.moveCenter(cr.center());
		p->setPen(itemg.highlightedText());
		p->drawPixmap(pmr.topLeft(), pixmap);
	}

	p->setPen(itemg.buttonText());
	QColor discol;
	if (dis) {
		discol = itemg.text();
		p->setPen(discol);
	}

	int xm = motifItemFrame + checkcol + motifItemHMargin;

	QString s = mi->text();
	if (!s.isNull()) {
		int t = s.find('\t');
		int m = motifItemVMargin;
		const int text_flags =
		    AlignVCenter | ShowPrefix | DontClip | SingleLine;
		if (t >= 0) {
			if (dis) {
				p->setPen(itemg.highlightedText());
				p->drawText(x + w - tab -
					    windowsRightBorder -
					    motifItemHMargin -
					    motifItemFrame + 1, y + m + 1,
					    tab, h - 2 * m, text_flags,
					    s.mid(t + 1));
				p->setPen(discol);
			}
			if (act && enabled) {
				p->setPen(itemg.highlightedText());
			}
			p->drawText(x + w - tab - windowsRightBorder -
				    motifItemHMargin - motifItemFrame,
				    y + m, tab, h - 2 * m, text_flags,
				    s.mid(t + 1));
		}

		if (dis) {
			p->setPen(itemg.highlightedText());
			p->drawText(x + xm + 1, y + m + 1, w - xm + 1,
				    h - 2 * m, text_flags, s, t);
			p->setPen(discol);
		}
		if (act && enabled) {
			p->setPen(itemg.highlightedText());
		}
		p->drawText(x + xm, y + m, w - xm - tab + 1, h - 2 * m,
			    text_flags, s, t);
	} else if (mi->pixmap()) {
		QPixmap *pixmap = mi->pixmap();
		if (pixmap->depth() == 1)
			p->setBackgroundMode(OpaqueMode);
		p->drawPixmap(x + xm, y + motifItemFrame, *pixmap);
		if (pixmap->depth() == 1)
			p->setBackgroundMode(TransparentMode);
	}

	if (mi->popup()) {
		int dim = (h - 2 * motifItemFrame) / 2;
		if (act) {
			if (!dis) {
				discol = itemg.highlightedText();
				itemg.setColor(QColorGroup::ButtonText, discol);
			}

			drawArrow(p, RightArrow, FALSE,
				  x + w - motifArrowHMargin -
				  motifItemFrame - dim,
				  y + h / 2 - dim / 2, dim, dim, itemg,
				  TRUE);
		} else {
			drawArrow(p, RightArrow, FALSE,
				  x + w - motifArrowHMargin -
				  motifItemFrame - dim,
				  y + h / 2 - dim / 2, dim, dim, itemg,
				  mi->isEnabled());
		}
	}
}

int dotNETStyle::popupMenuItemHeight(bool, QMenuItem * mi,
				     const QFontMetrics & fm)
{
	if (mi->isSeparator())
		return 3;

	int h = 0;
	if (mi->pixmap())
		h = mi->pixmap()->height();

	if (mi->iconSet())
		h = QMAX(mi->iconSet()->
			 pixmap(QIconSet::Small,
				QIconSet::Normal).height(), h);

	h = QMAX(fm.height() + 6, h);

	// we want a minimum size of 16
	h = QMAX(h, 16);

	return h;
}

void dotNETStyle::drawKProgressBlock
    (QPainter * p,
     int x, int y, int w, int h, const QColorGroup & g, QBrush * fill) {
	KStyle::drawKProgressBlock(p, x, y, w, h, g, fill);
}

void dotNETStyle::drawPanel
    (QPainter * p,
     int x,
     int y,
     int w,
     int h,
     const QColorGroup & g,
     bool sunken, int /* lineWidth */, const QBrush * /* brush */) {
	p->save();
	if (sunken) {
		if (h > 50 && w > 50) {
			p->setPen (g.background());
			p->drawRect (x, y, w, h);
			p->setPen (g.background().light());
			p->drawRect (x+1, y+1, w-2, h-2);
			p->setPen (g.background().dark());
			p->drawLine (x+1, y+1, w-2, y+1);
			p->drawLine (x+1, y+1, x+1, h-2);
		} else {
			p->setPen(g.background().dark());
			p->drawLine(x, y, w, y);
			p->drawLine(x, y, x, h);
			p->setPen(g.background().light());
			p->drawLine(x + 1, y + 1, w - 2, y + 1);
			p->drawLine(x + 1, y + 1, x + 1, h - 2);
		}
	} else {
		p->setPen(g.background());
		p->drawRect(x + 1, y + 1, w - 2, h - 2);

		p->setPen(g.background().light());
		p->drawRect(x, y, w, h);
		p->setPen(g.background().dark());
		p->drawLine(w - 1, h - 1, w - 1, y);
		p->drawLine(w - 1, h - 1, x, h - 1);
	}
	p->restore();
//      KStyle::drawPanel(p, x, y, w, h, g, sunken, lineWidth, brush);
}

void dotNETStyle::drawPopupPanel
    (QPainter * p,
     int x,
     int y, int w, int h, const QColorGroup & g, int, const QBrush *) {
	p->setPen(g.shadow());
	p->setBrush(g.midlight());
	p->drawRect(x, y, w, h);
	p->fillRect(x+1, y+1, 23, h-2, g.mid());
}

void dotNETStyle::drawSeparator
    (QPainter * p,
     int x,
     int y,
     int w,
     int h,
     const QColorGroup & g, bool sunken, int lineWidth, int midLineWidth) {
	KStyle::drawSeparator(p, x, y, w, h, g, sunken, lineWidth,
			      midLineWidth);
}

void dotNETStyle::drawTab
    (QPainter * p, const QTabBar * tabBar, QTab * tab, bool selected) {
	KStyle::drawTab(p, tabBar, tab, selected);
}

void dotNETStyle::drawTabMask
    (QPainter * p, const QTabBar * tabBar, QTab * tab, bool selected) {
	KStyle::drawTabMask(p, tabBar, tab, selected);
}

void dotNETStyle::drawKickerHandle
    (QPainter * p,
     int x, int y, int w, int h, const QColorGroup & g, QBrush *) {
	int k;
	p->save();
	p->setPen(NoPen);
	p->fillRect(x, y, w, h, g.background());
	if (w < h)
	//	Must be a vertical handle.
		for (k = 4; k < h - 4; k += 3) {
			p->fillRect(x + 2, k, w - 4, 1, g.mid());
		}
	else
	//	Must be a horizontal handle.
		for (k = 4; k < w - 4; k += 3) {
			p->fillRect(k, y + 2, 1, h - 4, g.mid());
		}
	p->restore();
}

void dotNETStyle::drawKickerAppletHandle
    (QPainter * p,
     int x, int y, int w, int h, const QColorGroup & g, QBrush *) {
	int k;
	p->save();
	p->setPen(NoPen);
	p->fillRect(x, y, w, h, g.background());
	if (w < h)
	//	Must be a vertical handle.
		for (k = 4; k < h - 4; k += 3) {
			p->fillRect(x + 2, k, w - 4, 1, g.mid());
		}
	else
	//	Must be a horizontal handle.
		for (k = 4; k < w - 4; k += 3) {
			p->fillRect(k, y + 2, 1, h - 4, g.mid());
		}
	p->restore();
//  KStyle::drawKickerAppletHandle(p, x, y, w, h, g, brush);
}

void dotNETStyle::drawKickerTaskButton
    (QPainter * p,
     int x,
     int y,
     int w,
     int h,
     const QColorGroup & g,
     const QString &, bool active, QPixmap *, QBrush *) {
	p->save();
	p->setPen(g.shadow());
	p->drawRect(x, y, w, h);
	p->setPen(g.midlight());
	if (active) {
		p->drawLine(w, h, x, h);
		p->drawLine(w, h, w, y);
		p->fillRect(x + 1, y + 1, w - 2, h - 2, g.highlight());
	} else {
		p->drawLine(x, y, x, h);
		p->drawLine(x, y, w, y);
		p->fillRect(x + 1, y + 1, w - 2, h - 2, g.background());
	}
	p->restore();
//  KStyle::drawKickerTaskButton(p, x, y, w, h, g, title, active, icon, brush);
}

bool dotNETStyle::eventFilter(QObject * obj, QEvent * ev)
{

	if ( /*obj->inherits("QButton") || */ obj->inherits("QComboBox")) {
		if (ev->type() == QEvent::Enter) {
			QWidget *btn = (QWidget *) obj;
			if (btn->isEnabled()) {
				QPalette pal = btn->palette();
				pal.setColor(QColorGroup::Mid,
					     pal.active().
					     color(QColorGroup::
						   Highlight));
				pal.setColor(QColorGroup::Shadow,
					     pal.active().
					     color(QColorGroup::Highlight).
					     dark());
				btn->setPalette(pal);
			}
		} else if (ev->type() == QEvent::Leave) {
			QWidget *btn = (QWidget *) obj;
			QPalette pal = btn->palette();
			pal.setColor(QColorGroup::Mid,
				     pal.active().
				     color(QColorGroup::Background));
			pal.setColor(QColorGroup::Shadow,
				     pal.active().
				     color(QColorGroup::Light));
			btn->setPalette(pal);
		}
	}

	return false;
}

// vim: ts=8 sw=8
